@@ -55,7 +55,7 @@ def old_ns(ns):
 
 @python_2_unicode_compatible
 class Option(object):
-    """Decribes a Celery configuration option."""
+    """Describes a Celery configuration option."""
 
     alt = None
     deprecate_by = None
@@ -244,6 +244,7 @@ NAMESPACES = Namespace(
         short_lived_sessions=Option(
             False, type='bool', old={'celery_result_db_short_lived_sessions'},
         ),
+        table_schemas=Option(type='dict'),
         table_names=Option(type='dict', old={'celery_result_db_tablenames'}),
     ),
     task=Namespace(
@@ -318,6 +319,7 @@ NAMESPACES = Namespace(
         pool=Option(DEFAULT_POOL),
         pool_putlocks=Option(True, type='bool'),
         pool_restarts=Option(False, type='bool'),
+        proc_alive_timeout=Option(4.0, type='float'),
         prefetch_multiplier=Option(4, type='int'),
         redirect_stdouts=Option(
             True, type='bool', old={'celery_redirect_stdouts'},
