@@ -10,7 +10,7 @@ from sqlalchemy.pool import NullPool
 
 ResultModelBase = declarative_base()
 
-__all__ = ['SessionManager']
+__all__ = ('SessionManager',)
 
 
 def _after_fork_cleanup_session(session):
@@ -47,8 +47,7 @@ class SessionManager(object):
             if short_lived_sessions or dburi not in self._sessions:
                 self._sessions[dburi] = sessionmaker(bind=engine)
             return engine, self._sessions[dburi]
-        else:
-            return engine, sessionmaker(bind=engine)
+        return engine, sessionmaker(bind=engine)
 
     def prepare_models(self, engine):
         if not self.prepared:
