@@ -38,6 +38,8 @@ class ElasticsearchBackend(KeyValueStoreBackend):
     scheme = 'http'
     host = 'localhost'
     port = 9200
+    username = None
+    password = None
     es_retry_on_timeout = False
     es_timeout = 10
     es_max_retries = 3
@@ -50,10 +52,12 @@ class ElasticsearchBackend(KeyValueStoreBackend):
         if elasticsearch is None:
             raise ImproperlyConfigured(E_LIB_MISSING)
 
-        index = doc_type = scheme = host = port = None
+        index = doc_type = scheme = host = port = username = password = None
 
         if url:
-            scheme, host, port, _, _, path, _ = _parse_url(url)  # noqa
+            scheme, host, port, username, password, path, _ = _parse_url(url)  # noqa
+            if scheme == 'elasticsearch':
+                scheme = None
             if path:
                 path = path.strip('/')
                 index, _, doc_type = path.partition('/')
@@ -63,6 +67,8 @@ class ElasticsearchBackend(KeyValueStoreBackend):
         self.scheme = scheme or self.scheme
         self.host = host or self.host
         self.port = port or self.port
+        self.username = username or self.username
+        self.password = password or self.password
 
         self.es_retry_on_timeout = (
             _get('elasticsearch_retry_on_timeout') or self.es_retry_on_timeout
@@ -128,11 +134,16 @@ class ElasticsearchBackend(KeyValueStoreBackend):
 
     def _get_server(self):
         """Connect to the Elasticsearch server."""
+        http_auth = None
+        if self.username and self.password:
+            http_auth = (self.username, self.password)
         return elasticsearch.Elasticsearch(
             '%s:%s' % (self.host, self.port),
             retry_on_timeout=self.es_retry_on_timeout,
             max_retries=self.es_max_retries,
-            timeout=self.es_timeout
+            timeout=self.es_timeout,
+            scheme=self.scheme,
+            http_auth=http_auth,
         )
 
     @property
