@@ -25,6 +25,7 @@ from celery.five import PY3, python_2_unicode_compatible
 from celery.local import try_import
 from celery.result import GroupResult, allow_join_result
 from celery.utils import abstract
+from celery.utils.collections import ChainMap
 from celery.utils.functional import _regen
 from celery.utils.functional import chunks as _chunks
 from celery.utils.functional import (is_list, maybe_list, regen,
@@ -140,6 +141,7 @@ class Signature(dict):
         def _inner(subclass):
             cls.TYPES[name or subclass.__name__] = subclass
             return subclass
+
         return _inner
 
     @classmethod
@@ -193,6 +195,8 @@ class Signature(dict):
         """
         args = args if args else ()
         kwargs = kwargs if kwargs else {}
+        # Extra options set to None are dismissed
+        options = {k: v for k, v in options.items() if v is not None}
         # For callbacks: extra args are prepended to the stored args.
         args, kwargs, options = self._merge(args, kwargs, options)
         return self.type.apply(args, kwargs, **options)
@@ -214,6 +218,8 @@ class Signature(dict):
         """
         args = args if args else ()
         kwargs = kwargs if kwargs else {}
+        # Extra options set to None are dismissed
+        options = {k: v for k, v in options.items() if v is not None}
         try:
             _apply = self._apply_async
         except IndexError:  # pragma: no cover
@@ -234,7 +240,8 @@ class Signature(dict):
         options = options if options else {}
         if self.immutable and not force:
             return (self.args, self.kwargs,
-                    dict(self.options, **options) if options else self.options)
+                    dict(self.options,
+                         **options) if options else self.options)
         return (tuple(args) + tuple(self.args) if args else self.args,
                 dict(self.kwargs, **kwargs) if kwargs else self.kwargs,
                 dict(self.options, **options) if options else self.options)
@@ -265,6 +272,7 @@ class Signature(dict):
                                         app=self._app)
         signature._type = self._type
         return signature
+
     partial = clone
 
     def freeze(self, _id=None, group_id=None, chord=None,
@@ -298,6 +306,7 @@ class Signature(dict):
         # pylint: disable=too-many-function-args
         #   Borks on this, as it's a property.
         return self.AsyncResult(tid)
+
     _freeze = freeze
 
     def replace(self, args=None, kwargs=None, options=None):
@@ -392,10 +401,6 @@ class Signature(dict):
         # These could be implemented in each individual class,
         # I'm sure, but for now we have this.
         if isinstance(self, group):
-            if isinstance(other, group):
-                # group() | group() -> single group
-                return group(
-                    itertools.chain(self.tasks, other.tasks), app=self.app)
             # group() | task -> chord
             return chord(self, body=other, app=self._app)
         elif isinstance(other, group):
@@ -449,7 +454,8 @@ class Signature(dict):
 
         with app.producer_or_acquire(None) as producer:
             props = type.backend.on_task_call(producer, tid)
-            app.control.election(tid, 'task', self.clone(task_id=tid, **props),
+            app.control.election(tid, 'task',
+                                 self.clone(task_id=tid, **props),
                                  connection=producer.connection)
             return type.AsyncResult(tid)
 
@@ -506,6 +512,7 @@ class Signature(dict):
             return self.type.apply_async
         except KeyError:
             return _partial(self.app.send_task, self['task'])
+
     id = getitem_property('options.task_id', 'Task UUID')
     parent_id = getitem_property('options.parent_id', 'Task parent UUID.')
     root_id = getitem_property('options.root_id', 'Task root UUID.')
@@ -520,6 +527,63 @@ class Signature(dict):
         'immutable', 'Flag set if no longer accepts new arguments')
 
 
+def _prepare_chain_from_options(options, tasks, use_link):
+    # When we publish groups we reuse the same options dictionary for all of
+    # the tasks in the group. See:
+    # https://github.com/celery/celery/blob/fb37cb0b8/celery/canvas.py#L1022.
+    # Issue #5354 reported that the following type of canvases
+    # causes a Celery worker to hang:
+    # group(
+    #   add.s(1, 1),
+    #   add.s(1, 1)
+    # ) | tsum.s() | add.s(1) | group(add.s(1), add.s(1))
+    # The resolution of #5354 in PR #5681 was to only set the `chain` key
+    # in the options dictionary if it is not present.
+    # Otherwise we extend the existing list of tasks in the chain with the new
+    # tasks: options['chain'].extend(chain_).
+    # Before PR #5681 we overrode the `chain` key in each iteration
+    # of the loop which applies all the tasks in the group:
+    # options['chain'] = tasks if not use_link else None
+    # This caused Celery to execute chains correctly in most cases since
+    # in each iteration the `chain` key would reset itself to a new value
+    # and the side effect of mutating the key did not propagate
+    # to the next task in the group.
+    # Since we now mutated the `chain` key, a *list* which is passed
+    # by *reference*, the next task in the group will extend the list
+    # of tasks in the chain instead of setting a new one from the chain_
+    # variable above.
+    # This causes Celery to execute a chain, even though there might not be
+    # one to begin with. Alternatively, it causes Celery to execute more tasks
+    # that were previously present in the previous task in the group.
+    # The solution is to be careful and never mutate the options dictionary
+    # to begin with.
+    # Here is an example of a canvas which triggers this issue:
+    # add.s(5, 6) | group((add.s(1) | add.s(2), add.s(3))).
+    # The expected result is [14, 14]. However, when we extend the `chain`
+    # key the `add.s(3)` task erroneously has `add.s(2)` in its chain since
+    # it was previously applied to `add.s(1)`.
+    # Without being careful not to mutate the options dictionary, the result
+    # in this case is [16, 14].
+    # To avoid deep-copying the entire options dictionary every single time we
+    # run a chain we use a ChainMap and ensure that we never mutate
+    # the original `chain` key, hence we use list_a + list_b to create a new
+    # list.
+    if use_link:
+        return ChainMap({'chain': None}, options)
+    elif 'chain' not in options:
+        return ChainMap({'chain': tasks}, options)
+    elif tasks is not None:
+        # chain option may already be set, resulting in
+        # "multiple values for keyword argument 'chain'" error.
+        # Issue #3379.
+        # If a chain already exists, we need to extend it with the next
+        # tasks in the chain.
+        # Issue #5354.
+        # WARNING: Be careful not to mutate `options['chain']`.
+        return ChainMap({'chain': options['chain'] + tasks},
+                        options)
+
+
 @Signature.register_type(name='chain')
 @python_2_unicode_compatible
 class _chain(Signature):
@@ -600,10 +664,8 @@ class _chain(Signature):
             if link:
                 tasks[0].extend_list_option('link', link)
             first_task = tasks.pop()
-            # chain option may already be set, resulting in
-            # "multiple values for keyword argument 'chain'" error.
-            # Issue #3379.
-            options['chain'] = tasks if not use_link else None
+            options = _prepare_chain_from_options(options, tasks, use_link)
+
             first_task.apply_async(**options)
             return results[0]
 
@@ -673,10 +735,20 @@ class _chain(Signature):
                 # signature instead of a group.
                 tasks.pop()
                 results.pop()
-                task = chord(
-                    task, body=prev_task,
-                    task_id=prev_res.task_id, root_id=root_id, app=app,
-                )
+                try:
+                    task = chord(
+                        task, body=prev_task,
+                        task_id=prev_res.task_id, root_id=root_id, app=app,
+                    )
+                except AttributeError:
+                    # A GroupResult does not have a task_id since it consists
+                    # of multiple tasks.
+                    # We therefore, have to construct the chord without it.
+                    # Issues #5467, #3585.
+                    task = chord(
+                        task, body=prev_task,
+                        root_id=root_id, app=app,
+                    )
 
             if is_last_task:
                 # chain(task_id=id) means task id is set for the last task
@@ -796,7 +868,7 @@ class chain(_chain):
 
     Returns:
         ~celery.chain: A lazy signature that can be called to apply the first
-            task in the chain.  When that task succeeed the next task in the
+            task in the chain.  When that task succeeds the next task in the
             chain is applied, and so on.
     """
 
@@ -1118,7 +1190,7 @@ class group(Signature):
         try:
             gid = opts['task_id']
         except KeyError:
-            gid = opts['task_id'] = uuid()
+            gid = opts['task_id'] = group_id or uuid()
         if group_id:
             opts['group_id'] = group_id
         if chord:
@@ -1136,6 +1208,7 @@ class group(Signature):
         else:
             self.tasks = new_tasks
         return self.app.GroupResult(gid, results)
+
     _freeze = freeze
 
     def _freeze_unroll(self, new_tasks, group_id, chord, root_id, parent_id):
@@ -1293,6 +1366,8 @@ class chord(Signature):
             task = stack.popleft()
             if isinstance(task, group):
                 stack.extend(task.tasks)
+            elif isinstance(task, _chain) and isinstance(task.tasks[-1], group):
+                stack.extend(task.tasks[-1].tasks)
             else:
                 yield task if value is None else value
 
@@ -1321,8 +1396,7 @@ class chord(Signature):
         options.pop('chord', None)
         options.pop('task_id', None)
 
-        header.freeze(group_id=group_id, chord=body, root_id=root_id)
-        header_result = header(*partial_args, task_id=group_id, **options)
+        header_result = header.freeze(group_id=group_id, chord=body, root_id=root_id)
 
         if len(header_result) > 0:
             app.backend.apply_chord(
@@ -1332,6 +1406,7 @@ class chord(Signature):
                 countdown=countdown,
                 max_retries=max_retries,
             )
+            header_result = header(*partial_args, task_id=group_id, **options)
         # The execution of a chord body is normally triggered by its header's
         # tasks completing. If the header is empty this will never happen, so
         # we execute the body manually here.
