@@ -1,11 +1,13 @@
 """Create Celery app instances used for testing."""
 from __future__ import absolute_import, unicode_literals
+
 import weakref
 from contextlib import contextmanager
 from copy import deepcopy
+
 from kombu.utils.imports import symbol_by_name
-from celery import Celery
-from celery import _state
+
+from celery import Celery, _state
 
 #: Contains the default configuration values for the test app.
 DEFAULT_TEST_CONFIG = {
@@ -28,6 +30,10 @@ class Trap(object):
     """
 
     def __getattr__(self, name):
+        # Workaround to allow unittest.mock to patch this object
+        # in Python 3.8 and above.
+        if name == '_is_coroutine':
+            return None
         raise RuntimeError('Test depends on current_app')
 
 
