@@ -47,7 +47,7 @@ class ManagerMixin(object):
         return [res.id for res in r if res.id not in res.backend._cache]
 
     def wait_for(self, fun, catch,
-                 desc='thing', args=(), kwargs={}, errback=None,
+                 desc='thing', args=(), kwargs=None, errback=None,
                  max_retries=10, interval_start=0.1, interval_step=0.5,
                  interval_max=5.0, emit_warning=False, **options):
         # type: (Callable, Sequence[Any], str, Tuple, Dict, Callable,
@@ -57,6 +57,8 @@ class ManagerMixin(object):
         The `catch` argument specifies the exception that means the event
         has not happened yet.
         """
+        kwargs = {} if not kwargs else kwargs
+
         def on_error(exc, intervals, retries):
             interval = next(intervals)
             if emit_warning:
