@@ -183,7 +183,7 @@ class DjangoWorkerFixup(object):
     def _close_database(self):
         for conn in self._db.connections.all():
             try:
-                conn.close_if_unusable_or_obsolete()
+                conn.close()
             except self.interface_errors:
                 pass
             except self.DatabaseError as exc:
@@ -199,5 +199,5 @@ class DjangoWorkerFixup(object):
 
     def on_worker_ready(self, **kwargs):
         if self._settings.DEBUG:
-            warnings.warn('Using settings.DEBUG leads to a memory leak, never '
-                          'use this setting in production environments!')
+            warnings.warn('''Using settings.DEBUG leads to a memory
+            leak, never use this setting in production environments!''')
