@@ -74,10 +74,10 @@ def reset_state():
 
 def maybe_shutdown():
     """Shutdown if flags have been set."""
-    if should_stop is not None and should_stop is not False:
-        raise WorkerShutdown(should_stop)
-    elif should_terminate is not None and should_terminate is not False:
+    if should_terminate is not None and should_terminate is not False:
         raise WorkerTerminate(should_terminate)
+    elif should_stop is not None and should_stop is not False:
+        raise WorkerShutdown(should_stop)
 
 
 def task_reserved(request,
@@ -89,10 +89,12 @@ def task_reserved(request,
 
 
 def task_accepted(request,
-                  _all_total_count=all_total_count,
+                  _all_total_count=None,
                   add_active_request=active_requests.add,
                   add_to_total_count=total_count.update):
     """Update global state when a task has been accepted."""
+    if not _all_total_count:
+        _all_total_count = all_total_count
     add_active_request(request)
     add_to_total_count({request.name: 1})
     all_total_count[0] += 1
